@@ -1,5 +1,5 @@
 # Copyright (C) 2011  Chris Lalancette <clalance@redhat.com>
-# Copyright (C) 2012-2016  Chris Lalancette <clalancette@gmail.com>
+# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@ import os
 import re
 import shutil
 
+import oz.GuestFSManager
 import oz.Linux
 import oz.OzException
 import oz.ozutil
@@ -191,7 +192,7 @@ label customiso
         self.log.debug("Teardown step 1")
         # reset the authorized keys
         self.log.debug("Resetting authorized_keys")
-        self._guestfs_path_restore(g_handle, '/root/.ssh')
+        g_handle.path_restore('/root/.ssh')
 
     def _image_ssh_teardown_step_2(self, g_handle):
         """
@@ -200,12 +201,12 @@ label customiso
         self.log.debug("Teardown step 2")
         # remove custom sshd_config
         self.log.debug("Resetting sshd_config")
-        self._guestfs_path_restore(g_handle, '/etc/ssh/sshd_config')
+        g_handle.path_restore('/etc/ssh/sshd_config')
 
         # reset the service link
         self.log.debug("Resetting sshd service")
         if self.ssh_startuplink:
-            self._guestfs_path_restore(g_handle, self.ssh_startuplink)
+            g_handle.path_restore(self.ssh_startuplink)
 
     def _image_ssh_teardown_step_3(self, g_handle):
         """
@@ -214,16 +215,16 @@ label customiso
         self.log.debug("Teardown step 3")
         # remove announce cronjob
         self.log.debug("Resetting announcement to host")
-        self._guestfs_remove_if_exists(g_handle, '/etc/cron.d/announce')
+        g_handle.remove_if_exists('/etc/cron.d/announce')
 
         # remove reportip
         self.log.debug("Removing reportip")
-        self._guestfs_remove_if_exists(g_handle, '/root/reportip')
+        g_handle.remove_if_exists('/root/reportip')
 
         # reset the service link
         self.log.debug("Resetting cron service")
         if self.cron_startuplink:
-            self._guestfs_path_restore(g_handle, self.cron_startuplink)
+            g_handle.path_restore(self.cron_startuplink)
 
     def _image_ssh_teardown_step_4(self, g_handle):
         """
@@ -236,7 +237,7 @@ label customiso
                   "/etc/ssh/ssh_host_rsa_key", "/etc/ssh/ssh_host_rsa_key.pub",
                   "/etc/ssh/ssh_host_ecdsa_key", "/etc/ssh/ssh_host_ecdsa_key.pub",
                   "/etc/ssh/ssh_host_key", "/etc/ssh/ssh_host_key.pub"]:
-            self._guestfs_remove_if_exists(g_handle, f)
+            g_handle.remove_if_exists(f)
 
     def _image_ssh_setup_step_1(self, g_handle):
         """
@@ -244,10 +245,10 @@ label customiso
         """
         # part 1; upload the keys
         self.log.debug("Step 1: Uploading ssh keys")
-        self._guestfs_path_backup(g_handle, '/root/.ssh')
+        g_handle.path_backup('/root/.ssh')
         g_handle.mkdir('/root/.ssh')
 
-        self._guestfs_path_backup(g_handle, '/root/.ssh/authorized_keys')
+        g_handle.path_backup('/root/.ssh/authorized_keys')
 
         self._generate_openssh_key(self.sshprivkey)
 
@@ -263,7 +264,7 @@ label customiso
             raise oz.OzException.OzException("ssh not installed on the image, cannot continue")
 
         self.ssh_startuplink = self._get_service_runlevel_link(g_handle, 'ssh')
-        self._guestfs_path_backup(g_handle, self.ssh_startuplink)
+        g_handle.path_backup(self.ssh_startuplink)
         g_handle.ln_sf('/etc/init.d/ssh', self.ssh_startuplink)
 
         sshd_config_file = os.path.join(self.icicle_tmp, "sshd_config")
@@ -272,14 +273,14 @@ label customiso
         f.close()
 
         try:
-            self._guestfs_path_backup(g_handle, '/etc/ssh/sshd_config')
+            g_handle.path_backup('/etc/ssh/sshd_config')
             g_handle.upload(sshd_config_file, '/etc/ssh/sshd_config')
         finally:
             os.unlink(sshd_config_file)
 
     def _image_ssh_setup_step_3(self, g_handle):
         """
-        Fourth step for allowing remote access (make the guest announce itself
+        Third step for allowing remote access (make the guest announce itself
         on bootup).
         """
         # part 3; make sure the guest announces itself
@@ -314,7 +315,7 @@ label customiso
 
         self.cron_startuplink = self._get_service_runlevel_link(g_handle,
                                                                 'cron')
-        self._guestfs_path_backup(g_handle, self.cron_startuplink)
+        g_handle.path_backup(self.cron_startuplink)
         g_handle.ln_sf('/etc/init.d/cron', self.cron_startuplink)
 
     def _collect_setup(self, libvirt_xml):
@@ -323,7 +324,8 @@ label customiso
         """
         self.log.info("Collection Setup")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         # we have to do 3 things to make sure we can ssh into Debian
         # 1)  Upload our ssh key
@@ -351,7 +353,7 @@ label customiso
                 raise
 
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
 
     def _collect_teardown(self, libvirt_xml):
         """
@@ -359,7 +361,8 @@ label customiso
         """
         self.log.info("Collection Teardown")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         try:
             self._image_ssh_teardown_step_1(g_handle)
@@ -370,95 +373,9 @@ label customiso
 
             self._image_ssh_teardown_step_4(g_handle)
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
             shutil.rmtree(self.icicle_tmp)
 
-    def _internal_customize(self, libvirt_xml, action):
-        """
-        Internal method to customize and optionally generate an ICICLE for the
-        operating system after initial installation.
-        """
-        # the "action" input is actually a tri-state:
-        # action = "gen_and_mod" means to generate the icicle and to
-        #          potentially make modifications
-        # action = "gen_only" means to generate the icicle only, and not
-        #          look at any modifications
-        # action = "mod_only" means to not generate the icicle, but still
-        #          potentially make modifications
-
-        self.log.info("Customizing image")
-
-        if not self.tdl.packages and not self.tdl.files and not self.tdl.commands:
-            if action == "mod_only":
-                self.log.info("No additional packages, files, or commands to install, and icicle generation not requested, skipping customization")
-                return
-            elif action == "gen_and_mod":
-                # It is actually possible to get here with a "gen_and_mod"
-                # action but a TDL that contains no real customizations.
-                # In the "safe ICICLE" code below it is important to know
-                # when we are truly in a "gen_only" state so we modify
-                # the action here if we detect that ICICLE generation is the
-                # only task to be done.
-                # FIXME: See about doing this test earlier or in a more generic
-                # way
-                self.log.debug("Asked to gen_and_mod but no mods are present - changing action to gen_only")
-                action = "gen_only"
-
-        # when doing an oz-install with -g, this isn't necessary as it will
-        # just replace the port with the same port.  However, it is very
-        # necessary when doing an oz-customize since the serial port might
-        # not match what is specified in the libvirt XML
-        modified_xml = self._modify_libvirt_xml_for_serial(libvirt_xml)
-
-        if action == "gen_only" and self.safe_icicle_gen:
-            # We are only generating ICICLE and the user has asked us to do
-            # this without modifying the completed image by booting it.
-            # Create a copy on write snapshot to use for ICICLE
-            # generation - discard when finished
-            cow_diskimage = self.diskimage + "-icicle-snap.qcow2"
-            self._internal_generate_diskimage(force=True,
-                                              backing_filename=self.diskimage,
-                                              image_filename=cow_diskimage)
-            modified_xml = self._modify_libvirt_xml_diskimage(modified_xml, cow_diskimage, 'qcow2')
-
-        self._collect_setup(modified_xml)
-
-        icicle = None
-        try:
-            libvirt_dom = self.libvirt_conn.createXML(modified_xml, 0)
-
-            try:
-                guestaddr = None
-                guestaddr = self._wait_for_guest_boot(libvirt_dom)
-                self._test_ssh_connection(guestaddr)
-
-                if action == "gen_and_mod":
-                    self.do_customize(guestaddr)
-                    icicle = self.do_icicle(guestaddr)
-                elif action == "gen_only":
-                    icicle = self.do_icicle(guestaddr)
-                elif action == "mod_only":
-                    self.do_customize(guestaddr)
-                else:
-                    raise oz.OzException.OzException("Invalid customize action %s; this is a programming error" % (action))
-            finally:
-                if action == "gen_only" and self.safe_icicle_gen:
-                    # if this is a gen_only and safe_icicle_gen, there is no
-                    # reason to wait around for the guest to shutdown; we'll
-                    # be removing the overlay file anyway.  Just destroy it
-                    libvirt_dom.destroy()
-                else:
-                    self._shutdown_guest(guestaddr, libvirt_dom)
-        finally:
-            if action == "gen_only" and self.safe_icicle_gen:
-                # no need to teardown because we simply discard the file
-                # containing those changes
-                os.unlink(cow_diskimage)
-            else:
-                self._collect_teardown(modified_xml)
-
-        return icicle
-
     def _discover_repo_locality(self, repo_url, guestaddr, certdict):
         """
         Internal method to discover whether a repository is reachable from the
@@ -478,34 +395,9 @@ label customiso
             self.guest_execute_command(guestaddr, "echo '%s' > /etc/apt/sources.list.d/%s" % (repo.url.strip('\'"'), repo.name + "list"))
             self.guest_execute_command(guestaddr, "apt-get update")
 
-    def do_customize(self, guestaddr):
-        """
-        Method to customize by installing additional packages and files.
-        """
-        if not self.tdl.packages and not self.tdl.files and not self.tdl.commands:
-            # no work to do, just return
-            return
-
-        self._customize_repos(guestaddr)
-
-        self.log.debug("Installing custom packages")
-        packstr = ''
-        for package in self.tdl.packages:
-            packstr += package.name + ' '
-
-        if packstr != '':
-            self.guest_execute_command(guestaddr,
-                                       'apt-get install -y %s' % (packstr),
-                                       tunnels=None)
-
-        self._customize_files(guestaddr)
-
-        self.log.debug("Running custom commands")
-        for cmd in self.tdl.commands:
-            self.guest_execute_command(guestaddr, cmd.read())
-
-        self.log.debug("Syncing")
-        self.guest_execute_command(guestaddr, 'sync')
+    def _install_packages(self, guestaddr, packstr):
+        self.guest_execute_command(guestaddr,
+                                   'apt-get install -y %s' % (packstr))
 
     def do_icicle(self, guestaddr):
         """
@@ -529,14 +421,6 @@ label customiso
 
         return self._output_icicle_xml(packages, self.tdl.description)
 
-    def guest_live_upload(self, guestaddr, file_to_upload, destination,
-                          timeout=10):
-        """
-        Method to copy a file to the live guest.
-        """
-        return oz.ozutil.scp_copy_file(guestaddr, self.sshprivkey,
-                                       file_to_upload, destination, timeout)
-
     def _get_kernel_from_txt_cfg(self, fetchurl):
         """
         Internal method to download and parse the txt.cfg file from a URL.  If
@@ -552,7 +436,7 @@ label customiso
             raise oz.OzException.OzException("Could not find %s" % (txtcfgurl))
 
         txtcfg = os.path.join(self.icicle_tmp, "txt.cfg")
-        self.log.debug("Going to write txt.cfg to %s" % (txtcfg))
+        self.log.debug("Going to write txt.cfg to %s", txtcfg)
         txtcfgfd = os.open(txtcfg, os.O_RDWR | os.O_CREAT | os.O_TRUNC)
         os.unlink(txtcfg)
         fp = os.fdopen(txtcfgfd)
@@ -580,7 +464,7 @@ label customiso
         if kernel is None or initrd is None:
             raise oz.OzException.OzException("Empty kernel or initrd")
 
-        self.log.debug("Returning kernel %s and initrd %s" % (kernel, initrd))
+        self.log.debug("Returning kernel %s and initrd %s", kernel, initrd)
         return (kernel, initrd)
 
     def _gzip_file(self, inputfile, outputmode):
@@ -606,7 +490,7 @@ label customiso
         Internal method to create a modified CPIO initrd
         """
         extrafname = os.path.join(self.icicle_tmp, "extra.cpio")
-        self.log.debug("Writing cpio to %s" % (extrafname))
+        self.log.debug("Writing cpio to %s", extrafname)
         cpiofiledict = {}
         cpiofiledict[preseedpath] = 'preseed.cfg'
         oz.ozutil.write_cpio(cpiofiledict, extrafname)
@@ -631,30 +515,40 @@ label customiso
             pass
 
         if kernel is None:
-            self.log.debug("Kernel was None, trying debian-installer/%s/linux" % (self.debarch))
+            self.log.debug("Kernel was None, trying debian-installer/%s/linux", self.debarch)
             # we couldn't find the kernel in the txt.cfg, so try a
             # hard-coded path
             kernel = "debian-installer/%s/linux" % (self.debarch)
         if initrd is None:
-            self.log.debug("Initrd was None, trying debian-installer/%s/initrd.gz" % (self.debarch))
+            self.log.debug("Initrd was None, trying debian-installer/%s/initrd.gz", self.debarch)
             # we couldn't find the initrd in the txt.cfg, so try a
             # hard-coded path
             initrd = "debian-installer/%s/initrd.gz" % (self.debarch)
 
-        self._get_original_media('/'.join([self.url.rstrip('/'),
-                                           kernel.lstrip('/')]),
-                                 self.kernelcache, force_download)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.kernelcache)
 
         try:
             self._get_original_media('/'.join([self.url.rstrip('/'),
-                                               initrd.lstrip('/')]),
-                                     self.initrdcache, force_download)
-        except:
-            os.unlink(self.kernelfname)
-            raise
+                                               kernel.lstrip('/')]),
+                                     fd, outdir, force_download)
 
-        # if we made it here, then we can copy the kernel into place
-        shutil.copyfile(self.kernelcache, self.kernelfname)
+            # if we made it here, then we can copy the kernel into place
+            shutil.copyfile(self.kernelcache, self.kernelfname)
+        finally:
+            os.close(fd)
+
+        (fd, outdir) = oz.ozutil.open_locked_file(self.initrdcache)
+
+        try:
+            try:
+                self._get_original_media('/'.join([self.url.rstrip('/'),
+                                                   initrd.lstrip('/')]),
+                                         fd, outdir, force_download)
+            except:
+                os.unlink(self.kernelfname)
+                raise
+        finally:
+            os.close(fd)
 
         try:
             preseedpath = os.path.join(self.icicle_tmp, "preseed.cfg")
