@@ -1,5 +1,5 @@
 # Copyright (C) 2013  harmw <harm@weites.com>
-# Copyright (C) 2013-2016  Chris Lalancette <clalancette@gmail.com>
+# Copyright (C) 2013-2017  Chris Lalancette <clalancette@gmail.com>
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,7 +27,7 @@ import oz.OzException
 
 class FreeBSD(oz.Guest.CDGuest):
     """
-    Class for FreeBSD 10.0 installation.
+    Class for FreeBSD 10.0, 10.1, 10.2, 10.3 and 11.0 installation.
     """
     def __init__(self, tdl, config, auto, output_disk, netdev, diskbus,
                  macaddress):
@@ -88,7 +88,7 @@ def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
     """
     Factory method for FreeBSD installs.
     """
-    if tdl.update in ["10.0"]:
+    if tdl.update in ["10.0", "10.1", "10.2", "10.3", "11.0",]:
         if netdev is None:
             netdev = 'virtio'
         if diskbus is None:
@@ -100,4 +100,4 @@ def get_supported_string():
     """
     Return supported versions as a string.
     """
-    return "FreeBSD: 10"
+    return "FreeBSD: 10, 11"
