@@ -33,7 +33,8 @@ class RHEL7Guest(oz.RedHat.RedHatLinuxCDYumGuest):
                  diskbus=None, macaddress=None):
         oz.RedHat.RedHatLinuxCDYumGuest.__init__(self, tdl, config, auto,
                                                  output_disk, netdev, diskbus,
-                                                 True, True, "cpio", macaddress)
+                                                 True, True, "cpio", macaddress,
+                                                 True)
 
     def _modify_iso(self):
         """
@@ -74,4 +75,4 @@ def get_supported_string():
     """
     Return supported versions as a string.
     """
-    return "RHEL 7: Beta, 0, 1"
+    return "RHEL 7: Beta, 0, 1, 2"
