@@ -1,5 +1,5 @@
 # Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
-# Copyright (C) 2012-2016  Chris Lalancette <clalancette@gmail.com>
+# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,9 +27,9 @@ try:
 except ImportError:
     import ConfigParser as configparser
 import gzip
-import guestfs
 
 import oz.Guest
+import oz.GuestFSManager
 import oz.Linux
 import oz.ozutil
 import oz.OzException
@@ -81,6 +81,8 @@ Subsystem	sftp	/usr/libexec/openssh/sftp-server
         if self.tdl.kernel_param:
             self.cmdline += " " + self.tdl.kernel_param
 
+        self.virtio_channel_name = None
+
     def _generate_new_iso(self):
         """
         Method to create a new ISO based on the modified CD/DVD.
@@ -166,7 +168,7 @@ label customiso
         self.log.debug("Teardown step 1")
         # reset the authorized keys
         self.log.debug("Resetting authorized_keys")
-        self._guestfs_path_restore(g_handle, '/root/.ssh')
+        g_handle.path_restore('/root/.ssh')
 
     def _image_ssh_teardown_step_2(self, g_handle):
         """
@@ -175,7 +177,7 @@ label customiso
         self.log.debug("Teardown step 2")
         # remove custom sshd_config
         self.log.debug("Resetting sshd_config")
-        self._guestfs_path_restore(g_handle, '/etc/ssh/sshd_config')
+        g_handle.path_restore('/etc/ssh/sshd_config')
 
         # reset the service link
         self.log.debug("Resetting sshd service")
@@ -184,7 +186,7 @@ label customiso
                 g_handle.rm('/etc/systemd/system/multi-user.target.wants/sshd.service')
         else:
             startuplink = self._get_service_runlevel_link(g_handle, 'sshd')
-            self._guestfs_path_restore(g_handle, startuplink)
+            g_handle.path_restore(startuplink)
 
     def _image_ssh_teardown_step_3(self, g_handle):
         """
@@ -193,7 +195,7 @@ label customiso
         self.log.debug("Teardown step 3")
         # reset iptables
         self.log.debug("Resetting iptables rules")
-        self._guestfs_path_restore(g_handle, '/etc/sysconfig/iptables')
+        g_handle.path_restore('/etc/sysconfig/iptables')
 
     def _image_ssh_teardown_step_4(self, g_handle):
         """
@@ -201,16 +203,15 @@ label customiso
         """
         self.log.debug("Teardown step 4")
         self.log.debug("Removing announcement to host")
-        self._guestfs_remove_if_exists(g_handle,
-                                       '/etc/NetworkManager/dispatcher.d/99-reportip')
+        g_handle.remove_if_exists('/etc/NetworkManager/dispatcher.d/99-reportip')
 
         # remove announce cronjob
         self.log.debug("Resetting announcement to host")
-        self._guestfs_remove_if_exists(g_handle, '/etc/cron.d/announce')
+        g_handle.remove_if_exists('/etc/cron.d/announce')
 
         # remove reportip
         self.log.debug("Removing reportip")
-        self._guestfs_remove_if_exists(g_handle, '/root/reportip')
+        g_handle.remove_if_exists('/root/reportip')
 
         # reset the service link
         self.log.debug("Resetting crond service")
@@ -219,14 +220,14 @@ label customiso
                 g_handle.rm('/etc/systemd/system/multi-user.target.wants/crond.service')
         else:
             startuplink = self._get_service_runlevel_link(g_handle, 'crond')
-            self._guestfs_path_restore(g_handle, startuplink)
+            g_handle.path_restore(startuplink)
 
     def _image_ssh_teardown_step_5(self, g_handle):
         """
         Fifth step to undo _image_ssh_setup (reset SELinux).
         """
         self.log.debug("Teardown step 5")
-        self._guestfs_path_restore(g_handle, "/etc/selinux/config")
+        g_handle.path_restore("/etc/selinux/config")
 
     def _image_ssh_teardown_step_6(self, g_handle):
         """
@@ -235,19 +236,16 @@ label customiso
         in /etc/ssh.  Since this image might be cached later on, this method
         removes those keys.
         """
-        for f in ["/etc/ssh/ssh_host_dsa_key", "/etc/ssh/ssh_host_dsa_key.pub",
-                  "/etc/ssh/ssh_host_rsa_key", "/etc/ssh/ssh_host_rsa_key.pub",
-                  "/etc/ssh/ssh_host_ecdsa_key", "/etc/ssh/ssh_host_ecdsa_key.pub",
-                  "/etc/ssh/ssh_host_key", "/etc/ssh/ssh_host_key.pub"]:
-            self._guestfs_remove_if_exists(g_handle, f)
+        for key in g_handle.glob_expand("/etc/ssh/*_key*"):
+            g_handle.remove_if_exists(key)
 
         # Remove any lease files; this is so that subsequent boots don't try
         # to connect to a DHCP server that is on a totally different network
         for lease in g_handle.glob_expand("/var/lib/dhclient/*.leases"):
-            g_handle.rm_f(lease)
+            g_handle.rm(lease)
 
         for lease in g_handle.glob_expand("/var/lib/NetworkManager/*.lease"):
-            g_handle.rm_f(lease)
+            g_handle.rm(lease)
 
     def _collect_teardown(self, libvirt_xml):
         """
@@ -255,7 +253,8 @@ label customiso
         """
         self.log.info("Collection Teardown")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         try:
             self._image_ssh_teardown_step_1(g_handle)
@@ -270,7 +269,7 @@ label customiso
 
             self._image_ssh_teardown_step_6(g_handle)
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
             shutil.rmtree(self.icicle_tmp)
 
     def _image_ssh_setup_step_1(self, g_handle):
@@ -279,10 +278,10 @@ label customiso
         """
         # part 1; upload the keys
         self.log.debug("Step 1: Uploading ssh keys")
-        self._guestfs_path_backup(g_handle, '/root/.ssh')
+        g_handle.path_backup('/root/.ssh')
         g_handle.mkdir('/root/.ssh')
 
-        self._guestfs_path_backup(g_handle, '/root/.ssh/authorized_keys')
+        g_handle.path_backup('/root/.ssh/authorized_keys')
 
         self._generate_openssh_key(self.sshprivkey)
 
@@ -305,7 +304,7 @@ label customiso
                                '/etc/systemd/system/multi-user.target.wants/sshd.service')
         else:
             startuplink = self._get_service_runlevel_link(g_handle, 'sshd')
-            self._guestfs_path_backup(g_handle, startuplink)
+            g_handle.path_backup(startuplink)
             g_handle.ln_sf('/etc/init.d/sshd', startuplink)
 
         sshd_config_file = os.path.join(self.icicle_tmp, "sshd_config")
@@ -313,7 +312,7 @@ label customiso
             f.write(self.sshd_config)
 
         try:
-            self._guestfs_path_backup(g_handle, '/etc/ssh/sshd_config')
+            g_handle.path_backup('/etc/ssh/sshd_config')
             g_handle.upload(sshd_config_file, '/etc/ssh/sshd_config')
         finally:
             os.unlink(sshd_config_file)
@@ -324,7 +323,7 @@ label customiso
         """
         # part 3; open up iptables
         self.log.debug("Step 3: Open up the firewall")
-        self._guestfs_path_backup(g_handle, '/etc/sysconfig/iptables')
+        g_handle.path_backup('/etc/sysconfig/iptables')
 
     def _image_ssh_setup_step_4(self, g_handle):
         """
@@ -334,6 +333,11 @@ label customiso
         # part 4; make sure the guest announces itself
         self.log.debug("Step 4: Guest announcement")
 
+        if self.tdl.arch in ['ppc64', 'ppc64le']:
+            announce_device = '/dev/hvc1'
+        else:
+            announce_device = '/dev/ttyS1'
+
         scriptfile = os.path.join(self.icicle_tmp, "script")
 
         if g_handle.exists("/etc/NetworkManager/dispatcher.d"):
@@ -342,9 +346,9 @@ label customiso
 #!/bin/bash
 
 if [ "$1" = "eth0" -a "$2" = "up" ]; then
-    echo -n "!$DHCP4_IP_ADDRESS,%s!" > /dev/ttyS1
+    echo -n "!$DHCP4_IP_ADDRESS,%s!" > %s
 fi
-""" % (self.uuid))
+""" % (self.uuid, announce_device))
 
             try:
                 g_handle.upload(scriptfile,
@@ -364,8 +368,8 @@ DEV=$(/bin/awk '{if ($2 == 0) print $1}' /proc/net/route) &&
 [ -z "$DEV" ] && exit 0
 ADDR=$(/sbin/ip -4 -o addr show dev $DEV | /bin/awk '{print $4}' | /bin/cut -d/ -f1) &&
 [ -z "$ADDR" ] && exit 0
-echo -n "!$ADDR,%s!" > /dev/ttyS1
-""" % (self.uuid))
+echo -n "!$ADDR,%s!" > %s
+""" % (self.uuid, announce_device))
 
         try:
             g_handle.upload(scriptfile, '/root/reportip')
@@ -390,7 +394,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
                                '/etc/systemd/system/multi-user.target.wants/crond.service')
         else:
             startuplink = self._get_service_runlevel_link(g_handle, 'crond')
-            self._guestfs_path_backup(g_handle, startuplink)
+            g_handle.path_backup(startuplink)
             g_handle.ln_sf('/etc/init.d/crond', startuplink)
 
     def _image_ssh_setup_step_5(self, g_handle):
@@ -400,7 +404,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         # part 5; set SELinux to permissive mode so we don't have to deal with
         # incorrect contexts
         self.log.debug("Step 5: Set SELinux to permissive mode")
-        self._guestfs_path_backup(g_handle, '/etc/selinux/config')
+        g_handle.path_backup('/etc/selinux/config')
 
         selinuxfile = self.icicle_tmp + "/selinux"
         with open(selinuxfile, 'w') as f:
@@ -418,7 +422,8 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         """
         self.log.info("Collection Setup")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         # we have to do 5 things to make sure we can ssh into RHEL/Fedora:
         # 1)  Upload our ssh key
@@ -459,7 +464,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
                 raise
 
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
 
     def do_icicle(self, guestaddr):
         """
@@ -564,18 +569,10 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
             outf.writelines(inf)
             inf.close()
 
-            g = guestfs.GuestFS()
-            g.add_drive_opts(ext2file, format='raw')
-            self.log.debug("Launching guestfs")
-            g.launch()
-
-            g.mount_options('', g.list_devices()[0], "/")
-
-            g.upload(kspath, "/ks.cfg")
-
-            g.sync()
-            g.umount_all()
-            g.kill_subprocess()
+            g_handle = oz.GuestFSManager.GuestFS(ext2file, 'raw')
+            g_handle.mount_partitions()
+            g_handle.upload(kspath, "/ks.cfg")
+            g_handle.cleanup()
 
             # kickstart is added, lets recompress it
             oz.ozutil.gzip_create(ext2file, self.initrdfname)
@@ -606,7 +603,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
             # hard-coded path
             initrd = "images/pxeboot/initrd.img"
 
-        (fd, outdir) = self._open_locked_file(self.kernelcache)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.kernelcache)
 
         try:
             self._get_original_media('/'.join([self.url.rstrip('/'),
@@ -618,7 +615,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         finally:
             os.close(fd)
 
-        (fd, outdir) = self._open_locked_file(self.initrdcache)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.initrdcache)
 
         try:
             try:
@@ -702,12 +699,22 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         Method to run the operating system installation.
         """
         return self._do_install(timeout, force, 0, self.kernelfname,
-                                self.initrdfname, self.cmdline)
+                                self.initrdfname, self.cmdline, None,
+                                self.virtio_channel_name)
 
 class RedHatLinuxCDYumGuest(RedHatLinuxCDGuest):
     """
     Class for RedHat-based CD guests with yum support.
     """
+    def __init__(self, tdl, config, auto, output_disk, nicmodel, diskbus,
+                 iso_allowed, url_allowed, initrdtype, macaddress, use_yum):
+        oz.RedHat.RedHatLinuxCDGuest.__init__(self, tdl, config, auto,
+                                              output_disk, nicmodel, diskbus,
+                                              iso_allowed, url_allowed,
+                                              initrdtype, macaddress)
+
+        self.use_yum = use_yum
+
     def _check_url(self, iso=True, url=True):
         """
         Method to check if a URL specified by the user is one that will work
@@ -804,7 +811,10 @@ class RedHatLinuxCDYumGuest(RedHatLinuxCDGuest):
                 os.unlink(localname)
 
     def _install_packages(self, guestaddr, packstr):
-        self.guest_execute_command(guestaddr, 'yum -y install %s' % (packstr))
+        if self.use_yum:
+            self.guest_execute_command(guestaddr, 'yum -y install %s' % (packstr))
+        else:
+            self.guest_execute_command(guestaddr, 'dnf -y install %s' % (packstr))
 
     def _remove_repos(self, guestaddr):
         for repo in list(self.tdl.repositories.values()):
@@ -904,7 +914,7 @@ label customboot
                 return
 
         # name of the output file
-        (fd, outdir) = self._open_locked_file(self.orig_floppy)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.orig_floppy)
 
         try:
             self._get_original_floppy(self.url + "/images/bootnet.img", fd,
