@@ -1,5 +1,5 @@
 # Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
-# Copyright (C) 2012-2016  Chris Lalancette <clalancette@gmail.com>
+# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,13 +24,14 @@ import re
 import os
 import gzip
 
+import oz.GuestFSManager
 import oz.Linux
 import oz.ozutil
 import oz.OzException
 
 class UbuntuGuest(oz.Linux.LinuxCDGuest):
     """
-    Class for Ubuntu 5.04, 5.10, 6.06, 6.10, 7.04, 7.10, 8.04, 8.10, 9.04, 9.10, 10.04, 10.10, 11.04, 11.10, 12.04, 12.10, 13.04, 13.10, 14.04, 14.10, 15.04, and 15.10 installation.
+    Class for Ubuntu 5.04, 5.10, 6.06, 6.10, 7.04, 7.10, 8.04, 8.10, 9.04, 9.10, 10.04, 10.10, 11.04, 11.10, 12.04, 12.10, 13.04, 13.10, 14.04, 14.10, 15.04, 15.10, 16.04, 16.10, and 17.04  installation.
     """
     def __init__(self, tdl, config, auto, output_disk, initrd, nicmodel,
                  diskbus, macaddress):
@@ -74,6 +75,8 @@ Subsystem       sftp    /usr/libexec/openssh/sftp-server
                                         self.tdl.distro + self.tdl.update + self.tdl.arch + "-ramdisk")
 
         self.cmdline = "priority=critical locale=en_US"
+        if self.tdl.kernel_param:
+            self.cmdline += " " + self.tdl.kernel_param
 
         self.reboots = 0
         if self.tdl.update in ["5.04", "5.10"]:
@@ -172,7 +175,9 @@ PROMPT 0
                     kernelname = "/casper/vmlinuz"
                     if self.tdl.update in ["12.04.2", "12.04.3", "12.04.4",
                                            "12.04.5", "13.04", "13.10", "14.04",
-                                           "14.04.1", "14.10", "15.04", "15.10"] and self.tdl.arch == "x86_64":
+                                           "14.04.1", "14.04.2", "14.04.3", "14.04.4",
+                                           "14.04.5", "14.10", "15.04", "15.10",
+                                           "16.04", "16.04.1", "16.10", "17.04"] and self.tdl.arch == "x86_64":
                         kernelname += ".efi"
                     f.write("  kernel " + kernelname + "\n")
                     f.write("  append file=/cdrom/preseed/customiso.seed boot=casper automatic-ubiquity noprompt keyboard-configuration/layoutcode=us initrd=/casper/" + self.casper_initrd + "\n")
@@ -183,7 +188,6 @@ PROMPT 0
                     f.write("  kernel /install/vmlinuz\n")
                     f.write("  append preseed/file=/cdrom/preseed/customiso.seed debian-installer/locale=en_US " + keyboard + " netcfg/choose_interface=auto keyboard-configuration/layoutcode=us priority=critical initrd=/install/initrd.gz --\n")
 
-
     def get_auto_path(self):
         """
         Method to create the correct path to the Ubuntu preseed files.
@@ -245,7 +249,7 @@ PROMPT 0
         self.log.debug("Teardown step 1")
         # reset the authorized keys
         self.log.debug("Resetting authorized_keys")
-        self._guestfs_path_restore(g_handle, '/root/.ssh')
+        g_handle.path_restore('/root/.ssh')
 
     def _image_ssh_teardown_step_2(self, g_handle):
         """
@@ -254,12 +258,12 @@ PROMPT 0
         self.log.debug("Teardown step 2")
         # remove custom sshd_config
         self.log.debug("Resetting sshd_config")
-        self._guestfs_path_restore(g_handle, '/etc/ssh/sshd_config')
+        g_handle.path_restore('/etc/ssh/sshd_config')
 
         # reset the service link
         self.log.debug("Resetting sshd service")
         if self.ssh_startuplink:
-            self._guestfs_path_restore(g_handle, self.ssh_startuplink)
+            g_handle.path_restore(self.ssh_startuplink)
 
     def _image_ssh_teardown_step_3(self, g_handle):
         """
@@ -268,19 +272,18 @@ PROMPT 0
         self.log.debug("Teardown step 3")
         # remove announce cronjob
         self.log.debug("Resetting announcement to host")
-        self._guestfs_remove_if_exists(g_handle,
-                                       '/etc/NetworkManager/dispatcher.d/99-reportip')
+        g_handle.remove_if_exists('/etc/NetworkManager/dispatcher.d/99-reportip')
 
-        self._guestfs_remove_if_exists(g_handle, '/etc/cron.d/announce')
+        g_handle.remove_if_exists('/etc/cron.d/announce')
 
         # remove reportip
         self.log.debug("Removing reportip")
-        self._guestfs_remove_if_exists(g_handle, '/root/reportip')
+        g_handle.remove_if_exists('/root/reportip')
 
         # reset the service link
         self.log.debug("Resetting cron service")
         if self.cron_startuplink:
-            self._guestfs_path_restore(g_handle, self.cron_startuplink)
+            g_handle.path_restore(self.cron_startuplink)
 
     def _image_ssh_teardown_step_4(self, g_handle):
         """
@@ -294,12 +297,12 @@ PROMPT 0
                   "/etc/ssh/ssh_host_ecdsa_key", "/etc/ssh/ssh_host_ecdsa_key.pub",
                   "/etc/ssh/ssh_host_ed25519_key", "/etc/ssh/ssh_host_ed25519_key.pub",
                   "/etc/ssh/ssh_host_key", "/etc/ssh/ssh_host_key.pub"]:
-            self._guestfs_remove_if_exists(g_handle, f)
+            g_handle.remove_if_exists(f)
 
         # Remove any lease files; this is so that subsequent boots don't try
         # to connect to a DHCP server that is on a totally different network
         for lease in g_handle.glob_expand("/var/lib/dhcp/*.leases"):
-            g_handle.rm_f(lease)
+            g_handle.rm(lease)
 
     def _image_ssh_setup_step_1(self, g_handle):
         """
@@ -307,10 +310,10 @@ PROMPT 0
         """
         # part 1; upload the keys
         self.log.debug("Step 1: Uploading ssh keys")
-        self._guestfs_path_backup(g_handle, '/root/.ssh')
+        g_handle.path_backup('/root/.ssh')
         g_handle.mkdir('/root/.ssh')
 
-        self._guestfs_path_backup(g_handle, '/root/.ssh/authorized_keys')
+        g_handle.path_backup('/root/.ssh/authorized_keys')
 
         self._generate_openssh_key(self.sshprivkey)
 
@@ -326,7 +329,7 @@ PROMPT 0
             raise oz.OzException.OzException("ssh not installed on the image, cannot continue")
 
         self.ssh_startuplink = self._get_service_runlevel_link(g_handle, 'ssh')
-        self._guestfs_path_backup(g_handle, self.ssh_startuplink)
+        g_handle.path_backup(self.ssh_startuplink)
         g_handle.ln_sf('/etc/init.d/ssh', self.ssh_startuplink)
 
         sshd_config_file = os.path.join(self.icicle_tmp, "sshd_config")
@@ -334,7 +337,7 @@ PROMPT 0
             f.write(self.sshd_config)
 
         try:
-            self._guestfs_path_backup(g_handle, '/etc/ssh/sshd_config')
+            g_handle.path_backup('/etc/ssh/sshd_config')
             g_handle.upload(sshd_config_file, '/etc/ssh/sshd_config')
         finally:
             os.unlink(sshd_config_file)
@@ -398,7 +401,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
 
         self.cron_startuplink = self._get_service_runlevel_link(g_handle,
                                                                 'cron')
-        self._guestfs_path_backup(g_handle, self.cron_startuplink)
+        g_handle.path_backup(self.cron_startuplink)
         g_handle.ln_sf('/etc/init.d/cron', self.cron_startuplink)
 
     def _collect_setup(self, libvirt_xml):
@@ -407,7 +410,8 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         """
         self.log.info("Collection Setup")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         # we have to do 3 things to make sure we can ssh into Ubuntu
         # 1)  Upload our ssh key
@@ -435,7 +439,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
                 raise
 
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
 
     def _collect_teardown(self, libvirt_xml):
         """
@@ -443,7 +447,8 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         """
         self.log.info("Collection Teardown")
 
-        g_handle = self._guestfs_handle_setup(libvirt_xml)
+        g_handle = oz.GuestFSManager.GuestFSLibvirtFactory(libvirt_xml, self.libvirt_conn)
+        g_handle.mount_partitions()
 
         try:
             self._image_ssh_teardown_step_1(g_handle)
@@ -454,7 +459,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
 
             self._image_ssh_teardown_step_4(g_handle)
         finally:
-            self._guestfs_handle_cleanup(g_handle)
+            g_handle.cleanup()
             shutil.rmtree(self.icicle_tmp)
 
     def _customize_repos(self, guestaddr):
@@ -552,7 +557,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
                 # there is a bit of asymmetry here in that OSs that support cpio
                 # archives have the initial initrdfname copied in the higher level
                 # function, but we delete it here.  OSs that don't support cpio,
-                # though, get the initrd created right here.  C'est le vie
+                # though, get the initrd created right here.  C'est la vie
                 os.unlink(self.initrdfname)
                 raise
 
@@ -596,7 +601,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
             # hard-coded path
             initrd = "ubuntu-installer/%s/initrd.gz" % (self.debarch)
 
-        (fd, outdir) = self._open_locked_file(self.kernelcache)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.kernelcache)
 
         try:
             self._get_original_media('/'.join([self.url.rstrip('/'),
@@ -608,7 +613,7 @@ echo -n "!$ADDR,%s!" > /dev/ttyS1
         finally:
             os.close(fd)
 
-        (fd, outdir) = self._open_locked_file(self.initrdcache)
+        (fd, outdir) = oz.ozutil.open_locked_file(self.initrdcache)
 
         try:
             try:
@@ -706,7 +711,9 @@ def get_class(tdl, config, auto, output_disk=None, netdev=None, diskbus=None,
     if tdl.update in ["9.10", "10.04", "10.04.1", "10.04.2", "10.04.3", "10.10",
                       "11.04", "11.10", "12.04", "12.04.1", "12.04.2",
                       "12.04.3", "12.04.4", "12.04.5", "12.10", "13.04",
-                      "13.10", "14.04", "14.04.1", "14.10", "15.04", "15.10"]:
+                      "13.10", "14.04", "14.04.1", "14.04.2", "14.04.3", "14.04.4",
+                      "14.04.5", "14.10", "15.04", "15.10",
+                      "16.04", "16.04.1", "16.10", "17.04"]:
         if netdev is None:
             netdev = 'virtio'
         if diskbus is None:
@@ -718,4 +725,4 @@ def get_supported_string():
     """
     Return supported versions as a string.
     """
-    return "Ubuntu: 5.04, 5.10, 6.06[.1,.2], 6.10, 7.04, 7.10, 8.04[.1,.2,.3,.4], 8.10, 9.04, 9.10, 10.04[.1,.2,.3], 10.10, 11.04, 11.10, 12.04[.1,.2,.3,.4,.5], 12.10, 13.04, 13.10, 14.04[.1], 14.10, 15.04, 15.10"
+    return "Ubuntu: 5.04, 5.10, 6.06[.1,.2], 6.10, 7.04, 7.10, 8.04[.1,.2,.3,.4], 8.10, 9.04, 9.10, 10.04[.1,.2,.3], 10.10, 11.04, 11.10, 12.04[.1,.2,.3,.4,.5], 12.10, 13.04, 13.10, 14.04[.1,.2,.3,.4,.5], 14.10, 15.04, 15.10, 16.04[.1], 16.10, 17.04"
