@@ -1,5 +1,5 @@
 # Copyright (C) 2010,2011  Chris Lalancette <clalance@redhat.com>
-# Copyright (C) 2012-2016  Chris Lalancette <clalancette@gmail.com>
+# Copyright (C) 2012-2017  Chris Lalancette <clalancette@gmail.com>
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,14 +22,13 @@ Miscellaneous utility functions.
 import os
 import random
 import subprocess
-import tempfile
 import errno
 import stat
 import shutil
-import pycurl
 import gzip
 import time
 import select
+import urllib
 try:
     import configparser
 except ImportError:
@@ -37,6 +36,13 @@ except ImportError:
 import collections
 import ftplib
 import struct
+import fcntl
+import logging
+import socket
+
+import lxml.etree
+import monotonic
+import requests
 
 def generate_full_auto_path(relative):
     """
@@ -425,7 +431,7 @@ def subprocess_check_output(*popenargs, **kwargs):
 
     if retcode:
         cmd = ' '.join(*popenargs)
-        raise SubprocessException("'%s' failed(%d): %s" % (cmd, retcode, stderr), retcode)
+        raise SubprocessException("'%s' failed(%d): %s" % (cmd, retcode, stderr+stdout), retcode)
 
     return (stdout, stderr, retcode)
 
@@ -674,12 +680,18 @@ def rmtree_and_sync(directory):
     cannot help if the system is otherwise very busy, but it does ensure that
     the problem is not self-inflicted.
     """
-    shutil.rmtree(directory)
-    fd = os.open(os.path.dirname(directory), os.O_RDONLY)
     try:
-        os.fsync(fd)
-    finally:
-        os.close(fd)
+        shutil.rmtree(directory)
+        fd = os.open(os.path.dirname(directory), os.O_RDONLY)
+        try:
+            os.fsync(fd)
+        finally:
+            os.close(fd)
+    except OSError as err:
+        if err.errno == 2:
+            pass
+        else:
+            raise
 
 def parse_config(config_file):
     """
@@ -699,7 +711,7 @@ def parse_config(config_file):
         # /etc/oz/oz.cfg.  If neither of those exist, we don't throw an error
         # but instead let Oz pick sane defaults internally.
         parsed = config.read(os.path.expanduser("~/.oz/oz.cfg"))
-        if len(parsed) == 0:
+        if len(parsed) == 0 and os.geteuid() == 0:
             config.read("/etc/oz/oz.cfg")
 
     return config
@@ -738,6 +750,61 @@ def default_screenshot_dir():
     """
     return os.path.join(default_data_dir(), "screenshots")
 
+class LocalFileAdapter(requests.adapters.BaseAdapter):
+    '''
+    This class implements an adapter for requests so we can properly deal with file://
+    local files.
+    '''
+    @staticmethod
+    def _chkpath(method, path):
+        """Return an HTTP status for the given filesystem path."""
+        if method.lower() in ('put', 'delete'):
+            return 501, "Not Implemented"  # TODO
+        elif method.lower() not in ('get', 'head', 'post'):
+            return 405, "Method Not Allowed"
+        elif os.path.isdir(path):
+            return 400, "Path Not A File"
+        elif not os.path.isfile(path):
+            return 404, "File Not Found"
+        elif not os.access(path, os.R_OK):
+            return 403, "Access Denied"
+        else:
+            return 200, "OK"
+
+    def send(self, req, **kwargs):
+        """Return the file specified by the given request
+
+        @type req: C{PreparedRequest}
+        @todo: Should I bother filling `response.headers` and processing
+               If-Modified-Since and friends using `os.stat`?
+        """
+        path = os.path.normcase(os.path.normpath(urllib.url2pathname(req.path_url)))
+        response = requests.Response()
+
+        response.status_code, response.reason = self._chkpath(req.method, path)
+        if response.status_code == 200 and req.method.lower() != 'head':
+            try:
+                response.raw = open(path, 'rb')
+            except (OSError, IOError), err:
+                response.status_code = 500
+                response.reason = str(err)
+
+        if isinstance(req.url, bytes):
+            response.url = req.url.decode('utf-8')
+        else:
+            response.url = req.url
+
+        response.headers['Content-Length'] = os.path.getsize(path)
+        response.headers['Accept-Ranges'] = 'bytes'
+        response.headers['Redirect-URL'] = req.url
+        response.request = req
+        response.connection = self
+
+        return response
+
+    def close(self):
+        pass
+
 def http_get_header(url, redirect=True):
     """
     Function to get the HTTP headers from a URL.  The available headers will be
@@ -749,48 +816,15 @@ def http_get_header(url, redirect=True):
     'Redirect-URL' will always be None in the redirect=True case, and may be
     None in the redirect=True case if no redirects were required.
     """
-    info = {}
-    def _header(buf):
-        """
-        Internal function that is called back from pycurl perform() for
-        header data.
-        """
-        buf = buf.strip()
-        if len(buf) == 0:
-            return
-
-        split = buf.split(':')
-        if len(split) < 2:
-            # not a valid header; skip
-            return
-        key = split[0].strip()
-        value = split[1].strip()
-        info[key] = value
-
-    def _data(buf):
-        """
-        Empty function that is called back from pycurl perform() for body data.
-        """
-        pass
-
-    c = pycurl.Curl()
-    c.setopt(c.URL, url)
-    c.setopt(c.NOBODY, True)
-    c.setopt(c.HEADERFUNCTION, _header)
-    c.setopt(c.HEADER, True)
-    c.setopt(c.WRITEFUNCTION, _data)
-    if redirect:
-        c.setopt(c.FOLLOWLOCATION, True)
-    c.perform()
-    info['HTTP-Code'] = c.getinfo(c.HTTP_CODE)
-    if info['HTTP-Code'] == 0:
-        # if this was a file:/// URL, then the HTTP_CODE returned 0.
-        # set it to 200 to be compatible with http
-        info['HTTP-Code'] = 200
-    if not redirect:
-        info['Redirect-URL'] = c.getinfo(c.REDIRECT_URL)
-
-    c.close()
+    with requests.Session() as requests_session:
+        requests_session.mount('file://', LocalFileAdapter())
+        response = requests_session.post(url, allow_redirects=redirect, stream=True, timeout=10)
+        info = response.headers
+        info['HTTP-Code'] = response.status_code
+        if not redirect:
+            info['Redirect-URL'] = response.headers.get('Location')
+        else:
+            info['Redirect-URL'] = None
 
     return info
 
@@ -798,45 +832,18 @@ def http_download_file(url, fd, show_progress, logger):
     """
     Function to download a file from url to file descriptor fd.
     """
-    class Progress(object):
-        """
-        Internal class to represent progress on the connection.  This is only
-        required so that we have somewhere to store the "last_mb" variable
-        that is not global.
-        """
-        def __init__(self):
-            self.last_mb = -1
-
-        def progress(self, down_total, down_current, up_total, up_current):
-            """
-            Function that is called back from the pycurl perform() method to
-            update the progress information.
-            """
-            if down_total == 0:
-                return
-            current_mb = int(down_current) / 10485760
-            if current_mb > self.last_mb or down_current == down_total:
-                self.last_mb = current_mb
-                logger.debug("%dkB of %dkB" % (down_current/1024, down_total/1024))
-
-    def _data(buf):
-        """
-        Function that is called back from the pycurl perform() method to
-        actually write data to disk.
-        """
-        write_bytes_to_fd(fd, buf)
-
-    progress = Progress()
-    c = pycurl.Curl()
-    c.setopt(c.URL, url)
-    c.setopt(c.CONNECTTIMEOUT, 5)
-    c.setopt(c.WRITEFUNCTION, _data)
-    c.setopt(c.FOLLOWLOCATION, 1)
-    if show_progress:
-        c.setopt(c.NOPROGRESS, 0)
-        c.setopt(c.PROGRESSFUNCTION, progress.progress)
-    c.perform()
-    c.close()
+    with requests.Session() as requests_session:
+        requests_session.mount('file://', LocalFileAdapter())
+        response = requests_session.get(url, stream=True, allow_redirects=True,
+                                        headers={'Accept-Encoding': ''})
+        file_size = int(response.headers.get('Content-Length'))
+        chunk_size = 10*1024*1024
+        done = 0
+        for chunk in response.iter_content(chunk_size):
+            write_bytes_to_fd(fd, chunk)
+            done += len(chunk)
+            if show_progress:
+                logger.debug("%dkB of %dkB" % (done / 1024, file_size / 1024))
 
 def ftp_download_directory(server, username, password, basepath, destination):
     """
@@ -970,3 +977,142 @@ def recursively_add_write_bit(inputdir):
             except OSError as err:
                 if err.errno != errno.ENOENT:
                     raise
+
+def find_uefi_firmware(arch):
+    '''
+    A function to find the UEFI firmware file that corresponds to a certain
+    architecture.
+    '''
+    # Yuck.  Finding the UEFI firmware to start certain guests (like aarch64)
+    # is a really nasty process.  While slightly out of date, this blog post
+    # describes the mess: http://blog.wikichoon.com/2016/01/uefi-support-in-virt-install-and-virt.html
+    # Here, we replicate what libguestfs is doing here, which is to essentially
+    # hardcode paths where UEFI firmware can be found on popular distributions.
+    # I verified that these files exist on both Fedora/RHEL and Ubuntu.
+    # Hopefully there will be a nicer way to do this in the future.
+    class UEFI(object):
+        '''
+        A private class to hold the path to the loader and the path to the
+        NVRAM file.
+        '''
+        def __init__(self, loader, nvram):
+            self.loader = loader
+            self.nvram = nvram
+
+        def exists(self):
+            '''
+            A method that returns True if both the loader and the NVRAM files
+            exist, False otherwise.
+            '''
+            if os.path.exists(self.loader) and os.path.exists(self.nvram):
+                return True
+            return False
+
+    if arch in ['i386', 'i486', 'i586', 'i686']:
+        uefi_list = [UEFI('/usr/share/edk2.git/ovmf-ia32/OVMF_CODE-pure-efi.fd',
+                          '/usr/share/edk2.git/ovmf-ia32/OVMF_VARS-pure-efi.fd')]
+    elif arch in ['x86_64']:
+        uefi_list = [UEFI('/usr/share/OVMF/OVMF_CODE.fd',
+                          '/usr/share/OVMF/OVMF_VARS.fd'),
+                     UEFI('/usr/share/edk2.git/ovmf-x64/OVMF_CODE-pure-efi.fd',
+                          '/usr/share/edk2.git/ovmf-x64/OVMF_VARS-pure-efi.fd')]
+    elif arch in ['aarch64']:
+        uefi_list = [UEFI('/usr/share/AAVMF/AAVMF_CODE.fd',
+                          '/usr/share/AAVMF/AAVMF_VARS.fd'),
+                     UEFI('/usr/share/edk2/aarch64/QEMU_EFI-pflash.raw',
+                          '/usr/share/edk2/aarch64/vars-template-pflash.raw'),
+                     UEFI('/usr/share/edk2.git/aarch64/QEMU_EFI-pflash.raw',
+                          '/usr/share/edk2.git/aarch64/vars-template-pflash.raw')]
+    else:
+        raise Exception("Invalid arch for UEFI firmware")
+
+    for uefi in uefi_list:
+        if uefi.exists():
+            return uefi.loader, uefi.nvram
+
+    raise Exception("UEFI firmware is not installed!")
+
+def open_locked_file(filename):
+    """
+    A function to open and lock a file.  Returns a file descriptor referencing
+    the open and locked file.
+    """
+    outdir = os.path.dirname(filename)
+    mkdir_p(outdir)
+
+    fd = os.open(filename, os.O_RDWR|os.O_CREAT)
+
+    try:
+        fcntl.lockf(fd, fcntl.LOCK_EX)
+    except:
+        os.close(fd)
+        raise
+
+    return (fd, outdir)
+
+def lxml_subelement(root, name, text=None, attributes=None):
+    """
+    Function to add a new element to an LXML tree, optionally include text
+    and a dictionary of attributes.
+    """
+    tmp = lxml.etree.SubElement(root, name)
+    if text is not None:
+        tmp.text = text
+    if attributes is not None:
+        for k, v in attributes.items():
+            tmp.set(k, v)
+    return tmp
+
+def timed_loop(max_time, cb, msg, cb_arg=None):
+    '''
+    A function to deal with waiting for an event to occur.  Given a
+    maximum time to wait, a callback, and a message, it will wait until the maximum
+    time for the event to occur.  Each time through the loop, it will do the following:
+
+    1.  Check to see if it has been at least 10 seconds since it last logged.  If so, it
+        will log right now.
+    2.  Call the callback to check for the event.  If the callback returns True, the
+        loop quits immediately.  If it returns False, go on to step 3.
+    3.  Sleep for the portion of 1 second that was not taken up by the callback.
+
+    If the event occurred (the callback returned True), then this function returns
+    True.  If we timed out while waiting for the event to occur, this function returns
+    False.
+    '''
+    log = logging.getLogger('%s' % (__name__))
+    now = monotonic.monotonic()
+    end = now + max_time
+    next_print = now
+    while now < end:
+        now = monotonic.monotonic()
+        if now >= next_print:
+            log.debug("%s, %d/%d", msg, int(end) - int(now), max_time)
+            next_print = now + 10
+
+            if cb(cb_arg):
+                return True
+
+        # It's possible that the callback took longer than one second.
+        # In that case, just skip our sleep altogether in an attempt to
+        # catch up.
+        sleep_time = 1.0 - (monotonic.monotonic() - now)
+        if sleep_time > 0:
+            # Otherwise, sleep for a time.  Note that we try to maintain
+            # on our starting boundary, so we'll sleep less than a second
+            # here almost always.
+            time.sleep(sleep_time)
+
+    return False
+
+def get_free_port():
+    """
+    A function to find a free TCP port on the host.
+    """
+    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    # Bind to port 0 which will use a free socket to listen to.
+    sock.bind(("", 0))
+    listen_port = sock.getsockname()[1]
+    # Close the socket to free it up for libvirt
+    sock.close()
+
+    return listen_port
